use super::{Verifier, FingerPrintStore};
use rustls::ServerConfig;
use std::{net::TcpStream, sync::Arc};

#[derive(Debug)]
pub struct Builder<S: FingerPrintStore> {
    pub stream: Option<TcpStream>,
    pub verifier: Option<Verifier<S>>,
}

impl<S: FingerPrintStore> Default for Builder<S> {
    fn default() -> Self {
        Self {
            stream: None,
            verifier: None,
        }
    }
}

impl<S: FingerPrintStore + 'static> Builder<S> {
    pub fn new() -> Self {
        Self::default()
    }

    pub fn stream(mut self, stream: TcpStream) -> Self {
        self.stream = Some(stream);
        self
    }

    pub fn verifier(mut self, verifier: Verifier<S>) -> Self {
        self.verifier = Some(verifier);
        self
    }

    pub fn build(self) -> Result<super::Connection, rustls::Error> {
        let cfg = ServerConfig::builder()
            .with_safe_default_cipher_suites()
            .with_safe_default_kx_groups()
            .with_safe_default_protocol_versions()?
            .with_client_cert_verifier(Arc::new(self.verifier.unwrap()));
        todo!()
    }
}
