use {std::fmt, x509_parser::prelude::X509Error};

#[derive(Debug)]
/// Errors which can occur when fingerprinting a certificate
pub enum Error {
    Fmt,
    InvalidForDate,
    X509(X509Error),
}

impl fmt::Display for Error {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            Self::Fmt => write!(f, "FingerPrint: format error"),
            Self::InvalidForDate => write!(f, "FingerPrint: invalid for date"),
            Self::X509(e) => write!(f, "FingerPrint: {e}"),
        }
    }
}

impl std::error::Error for Error {
    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
        match self {
            Self::X509(e) => Some(e),
            _ => None,
        }
    }
}

impl From<fmt::Error> for Error {
    fn from(_value: fmt::Error) -> Self {
        Self::Fmt
    }
}

impl From<x509_parser::nom::Err<x509_parser::error::X509Error>> for Error {
    fn from(value: x509_parser::nom::Err<x509_parser::error::X509Error>) -> Self {
        Self::X509(value.into())
    }
}
