use crate::prelude::{Host, Mailbox, ParseMailboxError as Error};
use std::{fmt, str::FromStr};

#[cfg(feature = "serde")]
use serde::{Deserialize, Serialize};

#[derive(Debug, Clone, PartialEq)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
pub struct Mailuser {
    pub username: String,
    pub host: Host,
}

impl fmt::Display for Mailuser {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{}@{}", self.username, self.host,)
    }
}

impl FromStr for Mailuser {
    type Err = Error;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        if let Some((username, host)) = s.split_once('@') {
            if username.is_empty() {
                Err(Error::EmptyUser)
            } else if host.is_empty() {
                Err(Error::EmptyHost)
            } else {
                let username = username.to_string();
                let host = host.parse()?;
                Ok(Self { username, host })
            }
        } else {
            Err(Error::MissingSeparator)
        }
    }
}

impl From<Mailbox> for Mailuser {
    fn from(value: Mailbox) -> Self {
        Self {
            username: value.username,
            host: value.host,
        }
    }
}
