use std::{fmt::Display, str::FromStr};

#[cfg(feature = "serde")]
use serde::{Deserialize, Serialize};

#[derive(Clone, Debug, PartialEq)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
pub struct Link {
    pub url: String,
    pub display: Option<String>,
}

impl Display for Link {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match &self.display {
            Some(d) => write!(f, "=> {} {d}", self.url),
            None => write!(f, "=> {}", self.url),
        }
    }
}

impl FromStr for Link {
    type Err = super::Error;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        let Some(s) = s.strip_prefix("=> ") else {
            return Err(super::Error::MalformedLink);
        };
        if let Some((url, display)) = s.split_once(char::is_whitespace) {
            Ok(Self {
                url: url.to_string(),
                display: Some(display.to_string()),
            })
        } else {
            Ok(Self {
                url: s.to_string(),
                display: None,
            })
        }
    }
}
